@@ -21,15 +21,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
+#include <string.h>
 #include <libintl.h>
 #include <glib-object.h>
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkcheckbutton.h>
 #include <gtk/gtkcombo.h>
+#include <gtk/gtkfilesel.h>
 #include <gtk/gtkframe.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkhbox.h>
@@ -37,6 +35,7 @@
 #include <gtk/gtkhseparator.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkmenu.h>
+#include <gtk/gtkmain.h>
 #include <gtk/gtkmenuitem.h>
 #include <gtk/gtkmenushell.h>
 #include <gtk/gtkmessagedialog.h>
@@ -50,16 +49,22 @@
 #include <gtk/gtktreeview.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkvpaned.h>
+#include <gtk/gtkvseparator.h>
+#include <gtk/gtkwindow.h>
+#ifdef BUILD_WITH_GNOME
 #include <libgnomedb/gnome-db-browser.h>
+#endif
 #include <libgnomedb/gnome-db-grid.h>
-#include <libgnomedb/gnome-db-intl.h>
 #include <libgnomedb/gnome-db-util.h>
+#ifdef BUILD_WITH_GNOME
 #include <libgnomeui/gnome-file-entry.h>
-#include <libgnomeui/gnome-window-icon.h>
+#endif
+#include "gnome-db-i18n.h"
 
 /**
  * gnome_db_new_browser_widget
  */
+#ifdef BUILD_WITH_GNOME
 GtkWidget *
 gnome_db_new_browser_widget (void)
 {
@@ -70,6 +75,7 @@ gnome_db_new_browser_widget (void)
 
 	return browser;
 }
+#endif
 
 /**
  * gnome_db_new_button_widget
@@ -150,6 +156,7 @@ gnome_db_new_entry_widget (gint max_length, gboolean editable)
 	return entry;
 }
 
+#ifdef BUILD_WITH_GNOME
 /**
  * gnome_db_new_file_entry_widget
  */
@@ -163,6 +170,7 @@ gnome_db_new_file_entry_widget (const gchar *history_id, const gchar *title)
 
 	return entry;
 }
+#endif
 
 /**
  * gnome_db_new_frame_widget
@@ -298,6 +306,7 @@ gnome_db_new_option_menu_widget (void)
 	return option_menu;
 }
 
+#ifdef BUILD_WITH_GNOME
 /**
  * gnome_db_new_popup_menu
  */
@@ -314,6 +323,7 @@ gnome_db_new_popup_menu (GtkWidget *parent, GnomeUIInfo *ui_info, gpointer user_
 
         return popup_menu;
 }
+#endif
 
 /**
  * gnome_db_new_radio_button_widget
@@ -554,23 +564,23 @@ gnome_db_option_menu_get_selection (GtkOptionMenu *option_menu)
 void
 gnome_db_option_menu_set_selection (GtkOptionMenu *option_menu, const gchar *selection)
 {
-	GtkWidget *menu;
 	GList *l;
+	gint i;
+	GtkWidget *menu;
 
 	g_return_if_fail (GTK_IS_OPTION_MENU (option_menu));
+	g_return_if_fail (selection != NULL);
 
 	menu = gtk_option_menu_get_menu (option_menu);
-	if (!GTK_IS_MENU_SHELL (menu))
-		return;
 
-	for (l = GTK_MENU_SHELL (menu)->children; l != NULL; l = l->next) {
+	for (l = GTK_MENU_SHELL (menu)->children, i = 0; l != NULL; l = l->next, i++) {
 		const gchar *str;
 		GtkWidget *menu_item = GTK_WIDGET (l->data);
 
 		str = (const gchar *) g_object_get_data (G_OBJECT (menu_item),
 							 "GNOME:Database:MenuItemLabel");
 		if (str && !strcmp (str, selection)) {
-			gtk_menu_shell_activate_item (GTK_MENU_SHELL (menu), menu_item, TRUE);
+			gtk_option_menu_set_history (option_menu, i);
 			break;
 		}
 	}
@@ -702,6 +712,35 @@ gnome_db_text_set_text (GtkTextView *text, const gchar *contents, gint len)
 	gtk_text_buffer_set_text (gtk_text_view_get_buffer (text), contents, len);
 }
 
+/**
+ * gnome_db_select_file_dialog
+ */
+gchar *
+gnome_db_select_file_dialog (GtkWidget *parent, const gchar *title)
+{
+	GtkWidget *filesel;
+        gchar *res = NULL;
+
+        /* create dialog */
+        filesel = gtk_file_selection_new (title);
+	if (GTK_IS_WINDOW (parent)) {
+		gtk_window_set_transient_for (GTK_WINDOW (filesel),
+				              GTK_WINDOW (parent));
+	}
+        gtk_file_selection_show_fileop_buttons (GTK_FILE_SELECTION (filesel));
+
+        /* wait for selection from user */
+	if (gtk_dialog_run (GTK_DIALOG (filesel)) == GTK_RESPONSE_OK)
+		res = g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel)));
+        else
+		res = NULL;
+
+        /* clean up */
+        gtk_widget_destroy (filesel);
+
+        return res;
+}
+
 /**
  * gnome_db_yes_no_dialog
  */
@@ -711,6 +750,7 @@ gnome_db_yes_no_dialog (GtkWidget *parent, const gchar *format, ...)
 	va_list args;
         gchar sz[2048];
 	GtkWidget *dialog;
+	GdkPixbuf *icon;
 	gboolean retval = FALSE;
 
 	/* build the message string */
@@ -723,8 +763,12 @@ gnome_db_yes_no_dialog (GtkWidget *parent, const gchar *format, ...)
 					 GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_YES_NO, sz);
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Question"));
-	gnome_window_icon_set_from_file (GTK_WINDOW (dialog),
-					 LIBGNOMEDB_PIXMAPSDIR "/gnome-db.png");
+
+	icon = gdk_pixbuf_new_from_file (LIBGNOMEDB_ICONSDIR "/gnome-db.png", NULL);
+	if (icon) {
+		gtk_window_set_icon (GTK_WINDOW (dialog), icon);
+		g_object_unref (icon);
+	}
 
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
 		retval = TRUE;
@@ -755,20 +799,64 @@ gnome_db_show_error (const gchar *format, ...)
 	gtk_widget_destroy (dialog);
 }
 
-
-/*
- * This function has been stolen from glib/glib/gutils.c
+/**
+ * gnome_db_show_message
+ * @format: Format string.
+ *
+ * Displays an information message to the user.
  */
-G_CONST_RETURN gchar *
-_gnome_db_gettext (const gchar *str)
+void
+gnome_db_show_message (const gchar *format, ...)
 {
-	static gboolean _gnome_db_gettext_initialized = FALSE;
+	va_list args;
+        gchar sz[2048];
+	GtkWidget *dialog;
 
-	if (!_gnome_db_gettext_initialized) {
-		bindtextdomain(GETTEXT_PACKAGE, LIBGNOMEDB_LOCALEDIR);
-		_gnome_db_gettext_initialized = TRUE;
-	}
+	/* build the message string */
+        va_start (args, format);
+        vsprintf (sz, format, args);
+        va_end (args);
 
-	return dgettext (GETTEXT_PACKAGE, str);
+	/* create the error message dialog */
+	dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_INFO,
+					 GTK_BUTTONS_CLOSE, sz);
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
 }
 
+void
+gnome_db_push_cursor_busy (GtkWidget *window)
+{
+        int busy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "GnomeDb:WindowBusy"));
+                                                                                
+        busy ++;
+                                                                                
+        if (busy == 1) {
+                gtk_widget_set_sensitive (window, FALSE);
+                if (window->window != NULL) {
+                        GdkCursor *cursor = gdk_cursor_new (GDK_WATCH);
+                        gdk_window_set_cursor (window->window, cursor);
+                        gdk_cursor_unref (cursor);
+                        gdk_flush ();
+                }
+        }
+                                                                                
+        g_object_set_data (G_OBJECT (window), "GnomeDb:WindowBusy", GINT_TO_POINTER (busy));
+}
+                                                                                
+void
+gnome_db_pop_cursor_busy (GtkWidget *window)
+{
+        int busy = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (window), "GnomeDb:WindowBusy"));
+        busy --;
+                                                                                
+        if (busy <= 0) {
+                gtk_widget_set_sensitive (window, TRUE);
+                if (window->window != NULL)
+                        gdk_window_set_cursor (window->window, NULL);
+                g_object_set_data (G_OBJECT (window),
+                                   "Panel:WindowBusy", NULL);
+        } else {
+                g_object_set_data (G_OBJECT (window), "Panel:WindowBusy", GINT_TO_POINTER (busy));
+        }
+}
